from tic.shared.utils.util import message_chunk_batch
from uuid import uuid4


class SnsClient(object):
    def __init__(self, boto_session, boto_config=None, endpoint_url=None):
        """

        @param boto_session: Preexisting boto session
        @param boto_config: optional boto config for the sns client
        """
        self.sns_client = boto_session.client(
            "sns", config=boto_config, endpoint_url=endpoint_url
        )

    def publish(self, topic: str, message: str):
        """
        Publish given message to a topic

        @param topic: topic ARN
        @param message: message to publish
        """
        self.sns_client.publish(TopicArn=topic, Message=message)

    def publish_messages_batch(self, topic: str, messages: list):
        """
        Delivers a list of messages to the specified topic

        @param topic: url of sns topic
        @param messages: the list of PublishBatchRequestEntries items to send
        """

        # Batch based on sns send_message_batch limit of 10 records or 256 KB limit
        for batch in message_chunk_batch(
            messages, num_records_per_batch=10, max_message_size=256000
        ):
            entries = []
            for message in batch:
                entries.append({"Message": message, "Id": str(uuid4())})
            self.sns_client.publish_batch(
                TopicArn=topic, PublishBatchRequestEntries=entries
            )

    def publish_in_batches(self, topic: str, messages: list):
        for batch in message_chunk_batch(
            messages, num_records_per_batch=1000, max_message_size=240000
        ):
            records = "\n".join(batch)
            self.sns_client.publish(TopicArn=topic, Message=records)
