from datetime import datetime

from tic.shared.dto import Dto
from tic.shared.reputation.dtos.mfg_scan_result import MfgScanResult
from tic.shared.reputation.enums.mfg_ingestion_source import MfgIngestionSource
from tic.shared.utils.date_util import from_iso_8601, to_iso_8601, utc_now


class MfgIngestionRecord(Dto):
    encode_names = {
        "source": "Source",
        "feed_date": "FeedDate",
        "data": "Data",
    }
    encode_settings = {
        "source": lambda x: x.value,
        "query_date": lambda x: to_iso_8601(x),
        "data": lambda x: x.to_json(minimize=True),
    }

    decode_names = {v: k for k, v in encode_names.items()}

    decode_settings = {
        "source": lambda x: None if x is None else MfgIngestionSource(x),
        "feed_date": lambda x: utc_now() if x is None else from_iso_8601(x),
    }

    def __init__(
        self,
        source: MfgIngestionSource = None,
        data: MfgScanResult = None,
        feed_date: datetime = None,
    ):
        self.source = source
        self.data = data
        self.feed_date = feed_date or utc_now
