from datetime import datetime
from enum import IntEnum

from ...enums.analysis.stage import UbsAnalysisStage
from ...enums.analysis.status import UbsAnalysisStatus
from ....utils.date_util import from_iso_8601, to_iso_8601, utc_now
from ....dto import Dto


class UbsBinarySource(IntEnum):
    APC = 0
    PSC = 1
    TAU = 2


class UbsAnalysisStateUpdate(Dto):
    decode_settings = {
        "completion_date": lambda x: utc_now() if x is None else from_iso_8601(x),
        "name": lambda x: UbsAnalysisStage(x),
        "status": lambda x: UbsAnalysisStatus(x),
        "version": lambda x: tuple(int(y) for y in x.split(".")),
    }

    encode_settings = {
        "completion_date": lambda x=utc_now(): to_iso_8601(x),
        "name": lambda x: x.value,
        "status": lambda x: x.value,
        "version": lambda x: ".".join((str(y) for y in x)),
    }

    def __init__(
        self,
        sha256: str = None,
        name: UbsAnalysisStage = None,
        version: tuple = None,
        completion_date: datetime = None,
        status: UbsAnalysisStatus = UbsAnalysisStatus.IN_PROGRESS,
        s3_file_location: str = None,
        sha256_truncated: str = None,
        truncated: bool = None,
        magic: str = None,
        source: int = -1,
    ):
        self.sha256 = sha256
        self.name = name
        self.version = (
            tuple(int(y) for y in version.split("."))
            if isinstance(version, str)
            else version
        )
        self.completion_date = (
            completion_date if completion_date is not None else utc_now()
        )
        self.status = status
        self.s3_file_location = s3_file_location
        self.sha256_truncated = sha256_truncated
        self.truncated = truncated
        self.magic = magic
        self.source = source
