# Program imports
from .customer_metadata import CustomerMetadata

test_envs = ["NELAPJQ7", "79LSMMRN"]


class ApcUploadMetadata(CustomerMetadata):
    """
    DTO for tracking CbD customer that uploaded a file to Cb
    """

    def __init__(
        self,
        registration_id=None,
        device_id=None,
        test=False,
        sha256=None,
        org_key=None,
        sha256_truncated=None,
    ):
        super().__init__(
            test=test, sha256=sha256, sha256_truncated=sha256_truncated, org_key=org_key
        )
        self.device_id = (
            device_id if device_id else self._get_device_id(registration_id)
        )
        self.registration_id = registration_id

    def __setattr__(self, name, value):
        if name == "registration_id":
            self.device_id = self._get_device_id(value)
        return super().__setattr__(name, value)

    def _get_device_id(self, registration_id):
        if hasattr(self, "device_id") and self.device_id:
            return self.device_id
        if not registration_id:
            return None
        registration_parts = registration_id.split("-")
        return registration_parts[1] if len(registration_parts) > 1 else None

    def is_valid(self):
        """
        Checks the validity of the object.
        @return: whether or not the metadata obj is valid
        """

        return super().is_valid() and self.device_id is not None

    def is_test(self):
        return self.test or self.org_key in test_envs

    def get_log_data(self):
        """
        Returns a dictionary of log details
        """

        return {
            "sha256": self.sha256,
            "device_id": self.device_id,
            "org_key": self.org_key,
            "test": self.test,
        }
