from binascii import unhexlify
from sqlalchemy import (
    BigInteger,
    Boolean,
    CheckConstraint,
    Column,
    DateTime,
    ForeignKey,
    Index,
    Integer,
    LargeBinary,
    SmallInteger,
    String,
    text,
)
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base

from ...utils.hash_util import calculate_sha256

# Autogenerated by sqlacodegen https://pypi.python.org/pypi/sqlacodegen

Base = declarative_base()
metadata = Base.metadata


class CompanyName(Base):
    __tablename__ = "company_name"
    __table_args__ = (CheckConstraint("length(sha256_name) = 32"),)

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('company_name_id_seq'::regclass)"),
    )
    name = Column(String, nullable=False)
    sha256_name = Column(LargeBinary, nullable=False, unique=True)


class Copyright(Base):
    __tablename__ = "copyright"
    __table_args__ = (CheckConstraint("length(sha256_name) = 32"),)

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('copyright_id_seq'::regclass)"),
    )
    name = Column(String, nullable=False)
    sha256_name = Column(LargeBinary, nullable=False, unique=True)


class Cust(Base):
    __tablename__ = "cust"
    __table_args__ = (CheckConstraint("length(search_key) = 32"),)

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('cust_id_seq'::regclass)"),
    )
    search_key = Column(LargeBinary, nullable=False, unique=True)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))
    org_key = Column(String(128))

    @staticmethod
    def get_search_key(org_key=None):
        if org_key is None or not org_key:
            raise Exception("a valid org_key is required")

        return unhexlify(calculate_sha256(org_key).encode("utf-8"))


class CustDeviceLink(Base):
    __tablename__ = "cust_device_link"
    __table_args__ = (
        Index(
            "idx_cust_device_link_cust_ix_link_id_device_id",
            "cust_ix_link_id",
            "device_id",
            unique=True,
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('cust_device_link_id_seq'::regclass)"),
    )
    cust_ix_link_id = Column(ForeignKey("cust_ix_link.id"), nullable=False)
    device_id = Column(ForeignKey("device.id"), nullable=False)
    first_seen = Column(DateTime(True), nullable=False, server_default=text("now()"))

    cust_ix_link = relationship("CustIxLink")
    device = relationship("Device", lazy="joined")


class CustDeviceLink1(Base):
    __tablename__ = "cust_device_link_1"
    __table_args__ = (
        Index(
            "idx_cust_device_link_1_cust_ix_link_id_device_id",
            "cust_ix_link_id",
            "device_id",
            unique=True,
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('cust_device_link_id_seq'::regclass)"),
    )
    cust_ix_link_id = Column(ForeignKey("cust_ix_link.id"), nullable=False)
    device_id = Column(ForeignKey("device.id"), nullable=False)
    first_seen = Column(DateTime(True), nullable=False, server_default=text("now()"))

    cust_ix_link = relationship("CustIxLink")
    device = relationship("Device", lazy="joined")


class CustDeviceLink3(Base):
    __tablename__ = "cust_device_link_3"
    __table_args__ = (
        Index(
            "idx_cust_device_link_3_cust_ix_link_id_device_id",
            "cust_ix_link_id",
            "device_id",
            unique=True,
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('cust_device_link_id_seq'::regclass)"),
    )
    cust_ix_link_id = Column(ForeignKey("cust_ix_link.id"), nullable=False)
    device_id = Column(ForeignKey("device.id"), nullable=False)
    first_seen = Column(DateTime(True), nullable=False, server_default=text("now()"))

    cust_ix_link = relationship("CustIxLink")
    device = relationship("Device", lazy="joined")


class CustFilepath(Base):
    __tablename__ = "cust_filepath"
    __table_args__ = (
        Index(
            "idx_cust_filepath_cust_ix_link_id_filepath_id",
            "cust_ix_link_id",
            "filepath_id",
            unique=True,
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('cust_filepath_id_seq'::regclass)"),
    )
    cust_ix_link_id = Column(ForeignKey("cust_ix_link.id"), nullable=False)
    filepath_id = Column(ForeignKey("filepath.id"), nullable=False)
    first_seen = Column(DateTime(True), nullable=False, server_default=text("now()"))
    num_occurrences = Column(Integer, nullable=False, server_default=text("1"))
    date_modified = Column(DateTime(True), nullable=False, server_default=text("now()"))

    cust_ix_link = relationship("CustIxLink")
    filepath = relationship("Filepath", lazy="joined")


class CustIxLink(Base):
    __tablename__ = "cust_ix_link"
    __table_args__ = (
        Index(
            "idx_cust_ix_link_file_index_id_cust_id",
            "file_index_id",
            "cust_id",
            unique=True,
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('cust_ix_link_id_seq'::regclass)"),
    )
    file_index_id = Column(ForeignKey("file_index.id"), nullable=False)
    cust_id = Column(ForeignKey("cust.id"), nullable=False, index=True)
    date_created = Column(
        DateTime(True), nullable=False, server_default=text("now()"), index=True
    )

    cust = relationship("Cust")
    file_index = relationship("FileIndex", lazy="joined")


class Device(Base):
    __tablename__ = "device"
    __table_args__ = (
        Index(
            "idx_device_device_id_device_name", "device_id", "device_name", unique=True
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('device_id_seq'::regclass)"),
    )
    device_id = Column(BigInteger, nullable=False)
    device_name = Column(String(256), nullable=False)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))
    last_seen = Column(
        DateTime(True), nullable=True
    )  # Note: we dont want to set legacy devices to now()


class FileDetail(Base):
    __tablename__ = "file_details"
    __table_args__ = (
        CheckConstraint("length(md5) = 16"),
        CheckConstraint("length(sha256) = 32"),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('file_details_id_seq'::regclass)"),
    )
    file_index_id = Column(ForeignKey("file_index.id"), nullable=False, unique=True)
    sha256 = Column(LargeBinary, nullable=False)
    md5 = Column(LargeBinary, nullable=False)
    s3_key = Column(String(1024), nullable=False)
    size = Column(BigInteger, nullable=False)
    truncated = Column(Boolean, nullable=False, server_default=text("false"))
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))
    date_modified = Column(DateTime(True), nullable=False, server_default=text("now()"))
    test = Column(Boolean, nullable=False, server_default=text("false"))

    file_index = relationship("FileIndex", lazy="joined")


class FileDetailsCustLink(Base):
    __tablename__ = "file_details_cust_link"

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('file_details_cust_link_id_seq'::regclass)"),
    )
    file_details_id = Column(ForeignKey("file_details.id"), nullable=False, index=True)
    cust_id = Column(ForeignKey("cust.id"), nullable=False, index=True)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))

    cust = relationship("Cust")
    file_details = relationship("FileDetail", lazy="joined")


class FileDetailsMagicLink(Base):
    __tablename__ = "file_details_magic_link"

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('file_details_magic_link_id_seq'::regclass)"),
    )
    magic_id = Column(ForeignKey("magic.id"), nullable=False, index=True)
    file_details_id = Column(ForeignKey("file_details.id"), nullable=False, unique=True)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))

    file_details = relationship("FileDetail")
    magic = relationship("Magic")


class FileIndex(Base):
    __tablename__ = "file_index"
    __table_args__ = (
        CheckConstraint("length(md5) = 16"),
        CheckConstraint("length(sha256) = 32"),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('file_index_id_seq'::regclass)"),
    )
    sha256 = Column(LargeBinary, nullable=False, unique=True)
    md5 = Column(LargeBinary, nullable=False, index=True)
    size = Column(BigInteger, nullable=False)
    date_created = Column(
        DateTime(True), nullable=False, index=True, server_default=text("now()")
    )


class Filepath(Base):
    __tablename__ = "filepath"
    __table_args__ = (CheckConstraint("length(sha256_path) = 32"),)

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('filepath_id_seq'::regclass)"),
    )
    path = Column(String, nullable=False)
    sha256_path = Column(LargeBinary, nullable=False, unique=True)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))


class Magic(Base):
    __tablename__ = "magic"
    __table_args__ = (CheckConstraint("length(sha256_value) = 32"),)

    id = Column(
        Integer,
        primary_key=True,
        server_default=text("nextval('magic_id_seq'::regclass)"),
    )
    value = Column(String(1024), nullable=False)
    sha256_value = Column(LargeBinary, nullable=False, unique=True)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))


class FileDetailsPartnerLink(Base):
    __tablename__ = "file_details_partner_link"

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('file_details_partner_link_id_seq'::regclass)"),
    )
    partner_id = Column(ForeignKey("partner.id"), nullable=False, index=True)
    file_details_id = Column(ForeignKey("file_details.id"), nullable=False, index=True)
    response_state = Column(SmallInteger, nullable=False)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))
    date_modified = Column(DateTime(True), nullable=False, server_default=text("now()"))

    file_details = relationship("FileDetail")
    partner = relationship("Partner")


class Metadatum(Base):
    __tablename__ = "metadata"

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('metadata_id_seq'::regclass)"),
    )
    file_index_id = Column(ForeignKey("file_index.id"), nullable=False, unique=True)
    internal_name_id = Column(ForeignKey("program_name.id"), nullable=False, index=True)
    product_name_id = Column(ForeignKey("program_name.id"), nullable=False, index=True)
    company_name_id = Column(ForeignKey("company_name.id"), nullable=False, index=True)
    copyright_id = Column(ForeignKey("copyright.id"), nullable=False, index=True)
    trademark_id = Column(ForeignKey("copyright.id"), nullable=False, index=True)
    file_description = Column(String)
    file_version = Column(String)
    comments = Column(String)
    original_filename = Column(String)
    product_description = Column(String)
    product_version = Column(String)
    private_build = Column(String)
    special_build = Column(String)
    icon = Column(LargeBinary)
    os_type = Column(String)
    architecture = Column(SmallInteger)
    lang_id = Column(SmallInteger)
    charset_id = Column(SmallInteger)
    sensor_version = Column(String)
    internal_version = Column(SmallInteger)
    policy_version = Column(SmallInteger)

    company_name = relationship("CompanyName", lazy="joined", innerjoin=True)
    copyright = relationship(
        "Copyright",
        primaryjoin="Metadatum.copyright_id == Copyright.id",
        lazy="joined",
        innerjoin=True,
    )
    file_index = relationship("FileIndex", lazy="joined", innerjoin=True)
    internal_name = relationship(
        "ProgramName",
        primaryjoin="Metadatum.internal_name_id == ProgramName.id",
        lazy="joined",
        innerjoin=True,
    )
    product_name = relationship(
        "ProgramName",
        primaryjoin="Metadatum.product_name_id == ProgramName.id",
        lazy="joined",
        innerjoin=True,
    )
    trademark = relationship(
        "Copyright",
        primaryjoin="Metadatum.trademark_id == Copyright.id",
        lazy="joined",
        innerjoin=True,
    )


class Partner(Base):
    __tablename__ = "partner"

    id = Column(
        SmallInteger,
        primary_key=True,
        server_default=text("nextval('partner_id_seq'::regclass)"),
    )
    name = Column(String(128), nullable=False)
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))


class ProgramName(Base):
    __tablename__ = "program_name"
    __table_args__ = (CheckConstraint("length(sha256_name) = 32"),)

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('program_name_id_seq'::regclass)"),
    )
    name = Column(String, nullable=False)
    sha256_name = Column(LargeBinary, nullable=False, unique=True)


class Signature(Base):
    __tablename__ = "signature"
    __table_args__ = (
        Index(
            "idx_signature_publisher_name_issuer_name_sign_time",
            "digsig_publisher_name_id",
            "digsig_publisher_name_id",
            "digsig_sign_time",
            unique=True,
        ),
    )

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('signature_id_seq'::regclass)"),
    )
    digsig_publisher_name_id = Column(
        ForeignKey("company_name.id"), nullable=False, index=True
    )
    digsig_issuer_name_id = Column(
        ForeignKey("company_name.id"), nullable=False, index=True
    )
    digsig_sign_time = Column(DateTime(True))
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))

    digsig_issuer_name = relationship(
        "CompanyName",
        primaryjoin="Signature.digsig_issuer_name_id == CompanyName.id",
        lazy="joined",
    )
    digsig_publisher_name = relationship(
        "CompanyName",
        primaryjoin="Signature.digsig_publisher_name_id == CompanyName.id",
        lazy="joined",
    )


class SignatureResultLink(Base):
    __tablename__ = "signature_result_link"

    id = Column(
        BigInteger,
        primary_key=True,
        server_default=text("nextval('signature_result_link_id_seq'::regclass)"),
    )
    cust_ix_link_id = Column(ForeignKey("cust_ix_link.id"), nullable=False, index=True)
    signature_id = Column(ForeignKey("signature.id"), nullable=False, index=True)
    is_catalog_signature = Column(Boolean, nullable=False, server_default=text("false"))
    first_seen = Column(DateTime(True), nullable=False, server_default=text("now()"))
    num_occurrences = Column(Integer, nullable=False, server_default=text("1"))
    date_created = Column(DateTime(True), nullable=False, server_default=text("now()"))
    date_modified = Column(DateTime(True), nullable=False, server_default=text("now()"))
    sig_state = Column(SmallInteger, nullable=False, server_default=text("0"))

    cust_ix_link = relationship("CustIxLink")
    signature = relationship("Signature", lazy="joined")
