from ...ubs.util import hash_to_binary
from ...ubs.util import architecture_values


class StaticMetadataInsert(object):
    def __init__(
        self,
        sha256,
        os_type=None,
        architecture=None,
        lang_id=None,
        charset_id=None,
        internal_name=None,
        product_name=None,
        company_name=None,
        copyright=None,
        trademark=None,
        file_description=None,
        file_version=None,
        comments=None,
        original_filename=None,
        product_description=None,
        product_version=None,
        private_build=None,
        special_build=None,
        sensor_version=None,
        internal_version=None,
        policy_version=None,
    ):
        self.sha256 = sha256
        self.os_type = os_type
        self.architecture = (
            architecture_values[architecture]
            if isinstance(architecture, str)
            else architecture
        )
        self.lang_id = lang_id
        self.charset_id = charset_id
        self.internal_name = internal_name
        self.product_name = product_name
        self.company_name = company_name
        self.copyright = copyright
        self.trademark = trademark
        self.file_description = file_description
        self.file_version = file_version
        self.comments = comments
        self.original_filename = original_filename
        self.product_description = product_description
        self.product_version = product_version
        self.private_build = private_build
        self.special_build = special_build
        self.sensor_version = sensor_version
        self.internal_version = internal_version
        self.policy_version = policy_version

    def to_tuple(self):
        return (
            hash_to_binary(self.sha256),
            self.os_type,
            self.architecture,
            self.lang_id,
            self.charset_id,
            self.internal_name,
            self.product_name,
            self.company_name,
            self.copyright,
            self.trademark,
            self.file_description,
            self.file_version,
            self.comments,
            self.original_filename,
            self.product_description,
            self.product_version,
            self.private_build,
            self.special_build,
            self.sensor_version,
            self.internal_version,
            self.policy_version,
        )

    def __gt__(self, other):
        if self.internal_version > other.internal_version:
            return True
        if self.policy_version > other.policy_version:
            return True
        return False
