from io import BufferedIOBase
from magic import Magic


def get_file_magic(
    file_buffer: BufferedIOBase, chunk_size: int = 16777216, magic: Magic = None
):
    """
    Get file magic from a file_buffer

    :param file_buffer: File-like object to get magic from
    :param chunk_size: Number of bytes to use to calculate magic
    :param magic: Magic object to use to calculate the file magic.
                  Only necessary if using a custom magic file
    """
    if magic is None:
        magic = Magic()
    return magic.from_buffer(file_buffer[:chunk_size])


def is_invalid_type(sample_magic, valid_magic_types):
    invalid_file_type = True

    # If empty string - don't upload
    if not sample_magic:
        return invalid_file_type

    for prefix in valid_magic_types:
        if sample_magic.startswith(prefix):
            invalid_file_type = False

    return invalid_file_type


executable_types = {
    "PE32 executable",
    "PE32+ executable",
    "MS-DOS executable",
    "ELF",
    "Mach-O",
    "Unix Fast File system [v1] (little-endian), last mounted on",
    "Minix filesystem, V2",
}
